@@ -71,8 +71,8 @@ eigenizeCalls <- c( ## component-wise unarys valid for either Eigen array or mat
     makeCallList('pow', 'eigenize_cWiseByScalarArray'),
     makeCallList(c('asRow', 'asCol'), 'eigenize_asRowOrCol'),
     ## component-wise unarys valid only for only Eigen array
-    makeCallList(c('exp','log','cube','cwiseInverse','sin','cos','tan','asin','acos', '!', callsFromExternalUnaries), 'eigenize_cWiseUnaryArray'), 
-    
+    makeCallList(c('exp','log','cube','cwiseInverse','sin','cos','tan','asin','acos'), 'eigenize_cWiseUnaryArray'), 
+    makeCallList(callsFromExternalUnaries, 'eigenize_cWiseUnaryArray_external'), 
     ## component-wise binarys valid for either Eigen array or matrix, but the arguments must match
     ## Check on which if any of these can extend to a scalar on one side
     makeCallList(c('pmin','pmax'), 'eigenize_cWiseBinaryArray'),
@@ -108,7 +108,7 @@ eigenizeCalls <- c( ## component-wise unarys valid for either Eigen array or mat
 )
 
 eigenizeCallsBeforeRecursing <- c( ## These cannot be calls that trigger aliasRisk. getParam always triggers an intermediate so it should never need handling here
-    makeCallList(c('size','nimArr_dmnorm_chol', 'nimArr_dmvt_chol', 'nimArr_dwish_chol', 'nimArr_dinvwish_chol', 'nimArr_dcar_normal', 'nimArr_ddirch','calculate','calculateDiff','getLogProb', 'getParam', 'getBound', 'getNodeFunctionIndexedInfo', 'concatenateTemp', 'MAKE_FIXED_VECTOR', 'hardCodedVectorInitializer'), 'eigenize_doNotRecurse'),
+    makeCallList(c('size','nimArr_dmnorm_chol', 'nimArr_dmvt_chol', 'nimArr_dwish_chol', 'nimArr_dinvwish_chol', 'nimArr_dcar_normal', 'nimArr_dcar_proper', 'nimArr_ddirch','calculate','calculateDiff','getLogProb', 'getParam', 'getBound', 'getNodeFunctionIndexedInfo', 'concatenateTemp', 'MAKE_FIXED_VECTOR', 'hardCodedVectorInitializer'), 'eigenize_doNotRecurse'),
     list(coeffSetter = 'eigenize_coeffSetter',
          nfVar = 'eigenize_nfVar',
          chainedCall = 'eigenize_chainedCall',
@@ -404,6 +404,9 @@ eigenize_coeffSetter <- function(code, symTab, typeEnv, workEnv) {
     setupExprs
 }
 
+## This is used instead of promoteTypes only for logical operators
+## Then the argument types are promoted to match each other,
+## not the output type.
 promoteArgTypes <- function(code) {
     if(!(inherits(code$args[[1]], 'exprClass') & inherits(code$args[[2]], 'exprClass'))) return(NULL)
     a1type <- code$args[[1]]$type
@@ -435,6 +438,16 @@ promoteArgTypes <- function(code) {
     NULL
 }
 
+## Generally for Eigen operations, input type matches output type.
+## One exception is for logical operators, which are handled by
+##   promoteArgTypes instead.
+## Another exception is for external unary calls.  These are unary
+## operators like "logit" or "nimStep" that are not built in to Eigen.
+## In C++ are called via Eigen's generic unary operator system.  Casting
+## of those types is done naturally via C++ so is not needed here. Also
+## doing it here would cause a bug (and was responsible for Issue #617).
+## E.g. nimStep(-0.5) should be 0.  But if the -0.5 is cast to int before
+## calling nimStep, we get nimStep(static_cast<int>(-0.5)) = 1. Wrong.
 promoteTypes <- function(code) {
     resultType <- code$type
     for(i in seq_along(code$args)) {
@@ -513,6 +526,21 @@ eigenize_cWiseUnaryArray <- function(code, symTab, typeEnv, workEnv) {
     invisible(NULL)
 }
 
+## This is like eigenize_cWiseUnaryArray but it does different
+## casting of argument type.
+eigenize_cWiseUnaryArray_external <- function(code, symTab, typeEnv, workEnv) {
+    if(code$nDim == 0) return(NULL)
+    newName <- eigenizeTranslate[[code$name]]
+    if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
+    code$name <- newName
+    code$eigMatrix <- FALSE
+    if(length(code$args[[1]]$eigMatrix) == 0) stop(exprClassProcessingErrorMsg(code, 'Information for eigenizing was not complete,'), call. = FALSE)
+    if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    ## no casting is necessary and could yield wrong answer. See
+    ## comments above promotTypes.
+    invisible(NULL)
+}
+
 eigenize_cWiseUnaryMatrix <- function(code, symTab, typeEnv, workEnv) {
     if(code$nDim == 0) return(NULL)
     newName <- eigenizeTranslate[[code$name]]
@@ -642,7 +670,7 @@ eigenize_cWiseAddSub <- function(code, symTab, typeEnv, workEnv) {
 }
 
 eigenizeArrayize <- function(code) {
-    newExpr <- exprClass(name = 'eigArray', args = list(code), eigMatrix = FALSE,
+    newExpr <- exprClass$new(name = 'eigArray', args = list(code), eigMatrix = FALSE,
                          isName = FALSE, isCall = TRUE, isAssign = FALSE,
                          nDim = code$nDim, sizeExprs = code$sizeExprs, type = code$type,
                          caller = code$caller, callerArgID = code$callerArgID)
@@ -652,7 +680,7 @@ eigenizeArrayize <- function(code) {
 }
 
 eigenizeMatricize <- function(code) {
-    newExpr <- exprClass(name = 'eigMatrix', args = list(code), eigMatrix = TRUE,
+    newExpr <- exprClass$new(name = 'eigMatrix', args = list(code), eigMatrix = TRUE,
                          isName = FALSE, isCall = TRUE, isAssign = FALSE,
                          nDim = code$nDim, sizeExprs = code$sizeExprs, type = code$type,
                          caller = code$caller, callerArgID = code$callerArgID)
@@ -723,7 +751,6 @@ eigenize_nfVar <- function(code, symTab, typeEnv, workEnv) { ## A lot like eigen
     thisMapAlreadySet <- FALSE
     if(!is.null(workEnv[['OnLHSnow']])) { ## this is the var on the LHS
         if(!is.null(workEnv[['LHSeigenName']])) {
-            browser()
             stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE)
         }
         workEnv$LHSeigenName <- list(EigenName = EigenName, targetVar = targetVarProxy)
@@ -827,7 +854,6 @@ eigenizeName <- function(code, symTab, typeEnv, workEnv) {
     thisMapAlreadySet <- FALSE
     if(!is.null(workEnv[['OnLHSnow']])) { ## this is the var on the LHS
         if(!is.null(workEnv[['LHSeigenName']])) {
-            browser()
             stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE)
         }
         workEnv$LHSeigenName <- list(EigenName = EigenName, targetVar = code$name)
