@@ -1,3 +1,24 @@
+/*
+ * NIMBLE: an R package for programming with BUGS models.
+ * Copyright (C) 2014-2017 Perry de Valpine, Christopher Paciorek,
+ * Daniel Turek, Clifford Anderson-Bergman, Nick Michaud, Fritz Obermeyer,
+ * Duncan Temple Lang.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, a copy is available at
+ * https://www.R-project.org/Licenses/
+ */
+
 #include "nimble/accessorClasses.h"
 #include "nimble/RcppUtils.h"
 #include<sstream>
@@ -611,15 +632,15 @@ SEXP getListElement(SEXP list, const char *str){
 	return(ans);
 }
 
-SEXP populateNodeFxnVectorNew_byDeclID(SEXP SnodeFxnVec, SEXP S_GIDs, SEXP SnumberedObj, SEXP S_ROWINDS){
+void populateNodeFxnVectorNew_internal(NodeVectorClassNew* nfv, SEXP S_GIDs, SEXP SnumberedObj, SEXP S_ROWINDS ) {
   int len = LENGTH(S_ROWINDS);
-  if(len == 0) return(R_NilValue);
+  if(len == 0) return;
   int* gids = INTEGER(S_GIDs);
   int* rowinds = INTEGER(S_ROWINDS);
   int index;
   NumberedObjects* numObj = static_cast<NumberedObjects*>(R_ExternalPtrAddr(SnumberedObj));
-  NodeVectorClassNew* nfv = static_cast<NodeVectorClassNew*>(R_ExternalPtrAddr(SnodeFxnVec) ) ;
   int nextRowInd;
+  (*nfv).instructions.clear();
   for(int i = 0; i < len; i++){
     index = gids[i] - 1;
     nextRowInd = rowinds[i]-1;
@@ -628,23 +649,90 @@ SEXP populateNodeFxnVectorNew_byDeclID(SEXP SnodeFxnVec, SEXP S_GIDs, SEXP Snumb
     }
     (*nfv).instructions.push_back(NodeInstruction(static_cast<nodeFun*>(numObj->getObjectPtr(index)), nextRowInd));
   }
+}
+
+void populateNodeFxnVectorNew_copyFromRobject(void *nodeFxnVec_to, SEXP S_nodeFxnVec_from ) {
+  SEXP S_indexingInfo;
+   SEXP S_pxData;
+   PROTECT(S_pxData = Rf_allocVector(STRSXP, 1));
+   SET_STRING_ELT(S_pxData, 0, Rf_mkChar(".xData"));
+  // SEXP S_xData;
+  // PROTECT(S_xData = GET_SLOT(S_nodeFxnVec_from, S_pxData));
+  //PROTECT(S_indexingInfo = Rf_findVarInFrame(S_xData,
+  //					     Rf_install("indexingInfo")));
+  PROTECT(S_indexingInfo = VECTOR_ELT(S_nodeFxnVec_from, 1));
+  SEXP S_declIDs;
+  PROTECT(S_declIDs = VECTOR_ELT(S_indexingInfo, 0));
+  SEXP S_rowIndices;
+  PROTECT(S_rowIndices = VECTOR_ELT(S_indexingInfo, 1));
+  SEXP S_numberedPtrs;
+  // PROTECT(S_numberedPtrs = Rf_findVarInFrame(PROTECT(GET_SLOT(
+  // 							      Rf_findVarInFrame(PROTECT(GET_SLOT(
+  // 												 Rf_findVarInFrame(PROTECT(GET_SLOT(
+  // 																    Rf_findVarInFrame(S_xData,
+  // 																		      Rf_install("model")
+  // 																		      ),
+  // 																    S_pxData)),
+  // 														   Rf_install("CobjectInterface")
+  // 														   ),
+  // 												 S_pxData)),
+  // 										Rf_install(".nodeFxnPointers_byDeclID")),
+  // 							      S_pxData)),
+  // 					     Rf_install(".ptr")
+  // 					     )
+  // 	  );
+
+  // equivalent to S_nodeFxnVec_from[["model"]]$CobjectInterface$.nodeFxnPointers_byDeclID$.ptr
+  // implemented by S_nodeFxnVec_from[[2]]@.xData[["CobjectInterface"]]@.xData[[".nodeFxnPointers_byDeclID"]]@.xData[[".ptr"]]
+  PROTECT(S_numberedPtrs = Rf_findVarInFrame(PROTECT(GET_SLOT(
+							      Rf_findVarInFrame(PROTECT(GET_SLOT(
+												 Rf_findVarInFrame(PROTECT(GET_SLOT(
+																    VECTOR_ELT(S_nodeFxnVec_from,
+																	       2
+																	       ),
+																    S_pxData)),
+														   Rf_install("CobjectInterface")
+														   ),
+												 S_pxData)),
+										Rf_install(".nodeFxnPointers_byDeclID")),
+							      S_pxData)),
+					     Rf_install(".ptr")
+					     )
+	  );
+  NodeVectorClassNew* nfv = static_cast<NodeVectorClassNew*>(nodeFxnVec_to);
+  populateNodeFxnVectorNew_internal(nfv, S_declIDs, S_numberedPtrs, S_rowIndices);
+  UNPROTECT(8);
+}
+
+SEXP populateNodeFxnVectorNew_byDeclID(SEXP SnodeFxnVec, SEXP S_GIDs, SEXP SnumberedObj, SEXP S_ROWINDS){
+  NodeVectorClassNew* nfv = static_cast<NodeVectorClassNew*>(R_ExternalPtrAddr(SnodeFxnVec) ) ;
+  populateNodeFxnVectorNew_internal(nfv, S_GIDs, SnumberedObj, S_ROWINDS);
   return(R_NilValue);
 }
 
 SEXP populateIndexedNodeInfoTable(SEXP StablePtr, SEXP StableContents) {
   SEXP Sdim;
   PROTECT(Sdim = Rf_getAttrib(StableContents, R_DimSymbol));
-  if(LENGTH(Sdim) != 2) {PRINTF("Warning from populateIndexedNodeInfoTable: LENGTH(Sdim) != 2"); return(R_NilValue);}
+  if(LENGTH(Sdim) != 2) {
+    PRINTF("Warning from populateIndexedNodeInfoTable: LENGTH(Sdim) != 2");
+    UNPROTECT(1);
+    return R_NilValue;
+  }
   int nrow = INTEGER(Sdim)[0];
   int ncol = INTEGER(Sdim)[1];
   vector<indexedNodeInfo> *tablePtr = static_cast<vector<indexedNodeInfo> *>(R_ExternalPtrAddr(StablePtr));
   if(nrow == 0) {
     void *vptr=0;
     tablePtr->push_back(indexedNodeInfo(static_cast<int *>(vptr), 0, 0));
-    if(ncol != 0) {PRINTF("Warning from populateIndexedNodeInfoTable: nrow == 0 but ncol != 0.");}
+    if(ncol != 0) {
+      PRINTF("Warning from populateIndexedNodeInfoTable: nrow == 0 but ncol != 0.");
+    }
   } else {
-
-    if(!Rf_isNumeric(StableContents)) {PRINTF("Warning from populateIndexedNodeInfoTable: StableContents is not numeric"); return(R_NilValue);}
+    if(!Rf_isNumeric(StableContents)) {
+      PRINTF("Warning from populateIndexedNodeInfoTable: StableContents is not numeric");
+      UNPROTECT(1);
+      return R_NilValue;
+    }
     if(Rf_isInteger(StableContents)) {
       int *contentsPtr = INTEGER(StableContents);
       tablePtr->reserve(nrow);
@@ -660,7 +748,7 @@ SEXP populateIndexedNodeInfoTable(SEXP StablePtr, SEXP StableContents) {
     }
   }
   UNPROTECT(1);
-  return(R_NilValue);
+  return R_NilValue;
 }
 
 SEXP populateCopierVector(SEXP ScopierVector, SEXP SfromPtr, SEXP StoPtr, SEXP SintIsFromMV, SEXP SintIsToMV) {
