@@ -1,3 +1,24 @@
+/*
+ * NIMBLE: an R package for programming with BUGS models.
+ * Copyright (C) 2014-2017 Perry de Valpine, Christopher Paciorek,
+ * Daniel Turek, Clifford Anderson-Bergman, Nick Michaud, Fritz Obermeyer,
+ * Duncan Temple Lang.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, a copy is available at
+ * https://www.R-project.org/Licenses/
+ */
+
 #ifndef __DISTS
 #define __DISTS
 
@@ -47,6 +68,8 @@ extern "C" {
   SEXP C_dsqrtinvgamma(SEXP, SEXP, SEXP, SEXP);
   SEXP C_rsqrtinvgamma(SEXP, SEXP, SEXP);
   SEXP C_dcar_normal(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+  SEXP C_dcar_proper(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+  SEXP C_rcar_proper(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 }
 
 // NOTE: R CMD SHLIB seems to handle C++ code without using wrapping the functions in 'extern "C"'; note that some of these functions have a bit of C++ syntax
@@ -74,6 +97,9 @@ double rinterval(double, double*, int);
 double dcar_normal(double*, double*, double*, double*, double, int, int, int, int, int);
 void rcar_normal(int, double*, double*, double*, double, int, int);
 
+double dcar_proper(double*, double*, double*, double*, double*, double*, double, double, double*, int, int, int);
+void rcar_proper(double*, double*, double*, double*, double*, double*, double, double, double*, int, int);
+
 
 // SHOULD BE IN nimDists.h 
 // Chris comment on above line: I don't think this is the case...
