@@ -4,6 +4,11 @@
 
 source(system.file(file.path('tests', 'test_utils.R'), package = 'nimble'))
 
+RwarnLevel <- options('warn')$warn
+options(warn = 1)
+nimbleVerboseSetting <- nimbleOptions('verbose')
+nimbleOptions(verbose = FALSE)
+
 context("Testing of math functions in NIMBLE code")
 
 source(system.file(file.path('tests', 'mathTestLists.R'), package = 'nimble'))
@@ -17,8 +22,8 @@ if(.Platform$OS.type == 'windows') {
 }
 ans3 <- sapply(testsMoreMath, test_math, 'math')  ## 41
 ans4 <- sapply(testsReduction, test_math, 'math') ## 13
-ans5 <- sapply(testsComparison, test_math, 'math')## 6
+ans5 <- sapply(testsComparison, test_math, 'math')## 12
 ans6 <- sapply(testsMatrix, test_math, 'math')    ## 19
 
-
-
+options(warn = RwarnLevel)
+nimbleOptions(verbose = nimbleVerboseSetting)
